Module bit

Encodes a palette by bucketing bit ranges into a power of two number of buckets. This is very fast and produces ok results for most images at larger palette sizes (e.g. 256).

Structs

BitPaletteBuilder

Type Aliases

BitSixelEncoder4
BitSixelEncoder8
BitSixelEncoder16
BitSixelEncoder32
BitSixelEncoder64
BitSixelEncoder128
BitSixelEncoder256
BitSixelEncoderMono