BitPaletteBuilder

Struct BitPaletteBuilder 

pub struct BitPaletteBuilder { /* private fields */ }
Builds a palette by bucketing pixel colors via bit-dilation into a power-of-two number of buckets and averaging the colors in each bucket.

This is the fastest palette builder and produces acceptable results at larger palette sizes (e.g. 256).

Implementations

Source

impl BitPaletteBuilder

pub fn build_palette(image: &RgbaImage, palette_size: usize) -> Vec<Lab>

Quantize the image into palette_size colors using bit-dilation bucketing and return the resulting palette in Lab color space.