Struct Sobol

pub struct Sobol;
Uses the Sobol sequence to perturb the colors in a quasi-random manner. This is somewhere between ordered dithering and stochastic dithering. The results are generally decent, although they may appear textured like paper.