#[non_exhaustive]pub enum Dither {
None,
Sierra,
Sierra2,
SierraLite,
FloydSteinberg,
JJN,
Stucki,
Atkinson,
Burkes,
Bayer,
Sobol,
}Expand description
The dithering algorithm to apply during sixel encoding.
Choosing an algorithm
Dither::Sierrais a good default choice for dithering, as it produces high-quality results with minimal artifacts.Dither::Nonecan be used if performance is a concern.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
None
Do not perform dithering. This is the fastest possible palette encoding option.
Sierra
Sierra dithering (3-row).
Sierra2
Sierra-2 dithering (2-row).
SierraLite
Sierra Lite dithering.
FloydSteinberg
Floyd-Steinberg dithering.
JJN
Jarvis, Judice, and Ninke dithering.
Stucki
Stucki dithering.
Atkinson
Atkinson dithering.
Burkes
Burkes dithering.
Bayer
Bayer ordered dithering.
Sobol
Uses the Sobol sequence to perturb the colors in a quasi-random manner. This is somewhere between ordered dithering and stochastic dithering. The results are generally decent, although they may appear textured like paper.
Implementations
Sourceimpl Dither
impl Dither
Sourcepub fn dither_and_palettize(
&self,
image: &RgbaImage,
in_palette: &[Lab],
bucketer: &PaletteBucketer,
) -> Vec<usize>
pub fn dither_and_palettize( &self, image: &RgbaImage, in_palette: &[Lab], bucketer: &PaletteBucketer, ) -> Vec<usize>
Take the input image and convert it to the input palette, applying a dithering algorithm to the result.