pub struct SixelEncoder<P: PaletteBuilder = FocalPaletteBuilder, D: Dither = Sierra> { /* private fields */ }Expand description
The main type for performing sixel encoding.
It is provided with two generic parameters:
- A
PaletteBuilderto generate a color palette from the input image (sixel only supports up to 256 colors). - A
Dithertype to apply dithering to the reduced color image before encoding it into sixel format.
A number of type aliases are provided for common configurations, such as
ADUSixelEncoder256, which uses the ADUPaletteBuilder with 256
colors.
Choosing a PaletteBuilder
ADUPaletteBuilderis a good default choice for minimizing the error across the image. For maximum color accuracy at the cost of speed, you can useADUSixelEncoder256High.FocalPaletteBuilderis a good choice if the image has highlights and other color details that ADU might squash, but is experimental and much slower. You can increase the number of steps to improve accuracy even futher, as is done byFocalSixelEncoder256High.- Other palette builders are available, but are likely to perform less well at image accuracy than either of these two.
Choosing a Dither
Implementations
Sourceimpl<P: PaletteBuilder, D: Dither> SixelEncoder<P, D>
impl<P: PaletteBuilder, D: Dither> SixelEncoder<P, D>